%% Integral image
%
%  Le wavelet di Haar applicate a una immagine per una determinata 
% posizione e scala producono come risposta la somma dei valori dei pixel
% contenuti in aree rettangolari, per poter calcolare rapidamente la
% risposta di una immagine ad una determinata wavelet di haar 
% indispensabile fornire un algoritmo che calcoli questa somma in maniera
% efficiente, in particolare  indispensabile produrre un algoritmo che
% permetta di spostare la complessit computazionale dal calcolo di una
% singola risposta ad una operazione di preprocessing. L'integral image 
% uno immagine calcolabile in tempo lineare nel numero di pixel, tramite
% essa  possibile calcolare la somma di valori di pixel contenuti in
% rettangoli arbitrari in tempo costante (3 operazioni di somma/sottrazione
% in tutto). L'integral image di una immagine I  definita come segue:
%
%  II(x_0,y_0) = \sum_{x=0}^{x_0} \sum_{y=0}^{y_0} I(x,y)
%
%  In una dimensione tali somme cumulate sono calcolabili iterativamente
% seguendo la regola di ricorsione:
%
%  II(t) = \left\{ \begin{matrix} I(0) \hfill\quad\textbb{per\ } t=0 \\ I(t-1)+I(t) \hfill\quad\textbb{altrimenti} \end{matrix} \right.
%
%  In Matlab la funzione cumsum svolge questo compito. Per ottenere il
% risultato desiderato su una immagine cumsum deve essere applicata a tutte
% le righe e poi a tutte le colonne.

img = double(imread('test_3.jpg'))/255;

% Somma cumulata per righe e poi per colonne:
iimg = cumsum(img,2);
iimg = cumsum(iimg,1);

% Vediamola:
figure; imshowscale(iimg);hold on;

rect = [300,300,100,100]; plotrect(rect);


%% Vediamo ora come localizzare un oggetto all'interno di una immagine 
% attraverso la toolbox vision di matlab. Per prima cosa creiamo l'oggetto 
%capace di effettuare la detection e configuriamo alcuni parametri

catDetector = vision.CascadeObjectDetector('visionary.net_cat_cascade_web_LBP.xml');
catDetector.MinSize = [60 60];
catDetector.MergeThreshold = 2;

%% Effettiamo la detection
bboxCat = step(catDetector, img);
ICAT = insertObjectAnnotation(img, 'rectangle',bboxCat,'CAT');
figure, imshow(ICAT), title('Detecting cats');

% Notate la falsa detection. A cosa  dovuta?

%% Tariamo meglio i parametri e riproviamo.
catDetector.MergeThreshold = 6;
bboxCat = step(catDetector, img);
ICAT = insertObjectAnnotation(img, 'rectangle',bboxCat,'CAT');
figure, imshow(ICAT), title('Detecting cats');

%% Cambiamo modello e proviamo a localizzare un oggetto su una tipologia di
% immagine che non  stata inclusa nell'addestramento. Cosa sta succedendo?
catDetector = vision.CascadeObjectDetector('visionary.net_pedestrian_cascade_web_LBP.xml');
catDetector.MergeThreshold = 2;
bboxCat = step(catDetector, img);
ICAT = insertObjectAnnotation(img, 'rectangle',bboxCat,'PEDESTRIAN');
figure, imshow(ICAT), title('Detecting pedestrian');

%% Riproviamo su una categoria di immagini "sensate".
img = imread('test_4.jpg');
pedestrianDetector = vision.CascadeObjectDetector('visionary.net_pedestrian_cascade_web_LBP.xml');
bboxPedestrian = step(pedestrianDetector, img);
IPEDESTRIAN = insertObjectAnnotation(img, 'rectangle',bboxPedestrian,'PEDESTRIAN');
figure, imshow(IPEDESTRIAN), title('Detecting pedestrian');

%% Vediamo cosa sta localizzando il solo V&J eliminando il filtraggio finale
pedestrianDetector.MergeThreshold = 0;
bboxPedestrian = step(pedestrianDetector, img);
IPEDESTRIAN = insertObjectAnnotation(img, 'rectangle',bboxPedestrian,'PEDESTRIAN');
figure, imshow(IPEDESTRIAN), title('Detecting pedestrian');